package org.remote.roadhog;

import java.awt.Graphics2D;
import java.awt.Point;

public class GraphUtil
{

  /** 
   * Draw a filled arrowhead polygon at the second end of the given line (this
   * does not draw the line).<p>
   *
   * @param int    x1               first x coordinate of the line 
   * @param int    y1               first y coordinate of the line 
   * @param int    x2               second x coordinate of the line (arrow 
   *                                head point goes here)
   * @param int    y2               second y coordinate of the line (arrow 
   *                                head point goes here)
   * @param int    arrow_head_len   length of the sides of the arrow head
   * @param int    arrow_head_angle angle between each side and the line in 
   *                                degrees
   * @param double arrow_inset      percent inset for base of arrow [0..1] 
   *                                (values around 0.60 seem to work well).
   */
  public static void fill_arrowhead(Graphics2D g, int x1, int y1, int x2, int y2, 
    int arrow_head_len, int arrow_head_angle, double arrow_inset)
    {
      double dx, dy, len, sin_theta, cos_theta;
      double pt1_x, pt1_y, pt2_x, pt2_y, pt3_x, pt3_y;
      double ah_len = (double)arrow_head_len;
      double ah_angle = Math.PI*((double)arrow_head_angle)/180.0;
      int result1_x, result1_y, result2_x, result2_y, result3_x, result3_y;

      /* figure out the length of the line */
      dx = (double)(x2-x1);
      dy = (double)(y2-y1);
      len = Math.sqrt(dx*dx+dy*dy);

      /* bail out now if its zero length */
      if (len == 0) return;

      /* compute arrow head points if were were on line on x axis */
      pt1_x = len - ah_len*Math.cos(ah_angle);
      pt1_y = ah_len*Math.sin(ah_angle);
      pt2_x = len - (len-pt1_x)*arrow_inset;
      pt2_y = 0;
      pt3_x = pt1_x;
      pt3_y = -pt1_y;

      /* compute sin and cos of rotation to get line from x axis current pos */
      sin_theta = dy / len;
      cos_theta = dx / len;

      /* rotate and translate to get our final points */
      result1_x = (int)(pt1_x*cos_theta - pt1_y*sin_theta + 0.5) + x1;
      result1_y = (int)(pt1_x*sin_theta + pt1_y*cos_theta + 0.5) + y1;
      result2_x = (int)(pt2_x*cos_theta - pt2_y*sin_theta + 0.5) + x1;
      result2_y = (int)(pt2_x*sin_theta + pt2_y*cos_theta + 0.5) + y1;
      result3_x = (int)(pt3_x*cos_theta - pt3_y*sin_theta + 0.5) + x1;
      result3_y = (int)(pt3_x*sin_theta + pt3_y*cos_theta + 0.5) + y1;

      /* draw the arrow head polygon */
      int xs[] = {x2, result1_x, result2_x, result3_x};
      int ys[] = {y2, result1_y, result2_y, result3_y};
      g.fillPolygon (xs,ys,4);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Draw a filled arrowhead polygon (with good defaults) at the second end 
   * of the given line (this does not draw the line).
   *
   * @param int x1 first x coordinate of the line 
   * @param int y1 first y coordinate of the line 
   * @param int x2 second x coordinate of the line (arrow head point goes here)
   * @param int y2 second y coordinate of the line (arrow head point goes here)
   */
  public static void fill_arrowhead(Graphics2D g, int x1, int y1, int x2, int y2)
    {
      fill_arrowhead(g, x1,y1,x2,y2, 20, 15, 0.60);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Draw a hollow arrowhead (as lines) at the second end of the given line 
   * (this does not draw the line).<p>
   *
   * @param int    x1               first x coordinate of the line. 
   * @param int    y1               first y coordinate of the line.
   * @param int    x2               second x coordinate of the line (arrow 
   *                                head point goes here).
   * @param int    y2               second y coordinate of the line (arrow 
   *                                head point goes here).
   * @param int    arrow_head_len   length of the sides of the arrow head.
   * @param int    arrow_head_angle angle between each side and the line in 
   *                                degrees.
   * @param double arrow_inset      percent inset for base of arrow [0..1] 
   *                                (values around 0.60 seem to work well).
   */
  public static void draw_arrowhead(Graphics2D g, int x1, int y1, int x2, int y2, 
    int arrow_head_len, int arrow_head_angle, double arrow_inset)
    {
      double dx, dy, len, sin_theta, cos_theta;
      double pt1_x, pt1_y, pt2_x, pt2_y, pt3_x, pt3_y;
      double ah_len = (double)arrow_head_len;
      double ah_angle = Math.PI*((double)arrow_head_angle)/180.0;
      int result1_x, result1_y, result2_x, result2_y, result3_x, result3_y;

      /* figure out the length of the line */
      dx = (double)(x2-x1);
      dy = (double)(y2-y1);
      len = Math.sqrt(dx*dx+dy*dy);

      /* bail out now if its zero length */
      if (len == 0) return;

      /* compute arrow head points if were were on line on x axis */
      pt1_x = len - ah_len*Math.cos(ah_angle);
      pt1_y = ah_len*Math.sin(ah_angle);
      pt2_x = len - (len-pt1_x)*arrow_inset;
      pt2_y = 0;
      pt3_x = pt1_x;
      pt3_y = -pt1_y;

      /* compute sin and cos of rotation to get line from x axis current pos */
      sin_theta = dy / len;
      cos_theta = dx / len;

      /* rotate and translate to get our final points */
      result1_x = (int)(pt1_x*cos_theta - pt1_y*sin_theta + 0.5) + x1;
      result1_y = (int)(pt1_x*sin_theta + pt1_y*cos_theta + 0.5) + y1;
      result2_x = (int)(pt2_x*cos_theta - pt2_y*sin_theta + 0.5) + x1;
      result2_y = (int)(pt2_x*sin_theta + pt2_y*cos_theta + 0.5) + y1;
      result3_x = (int)(pt3_x*cos_theta - pt3_y*sin_theta + 0.5) + x1;
      result3_y = (int)(pt3_x*sin_theta + pt3_y*cos_theta + 0.5) + y1;

      /* draw the arrow head lines */
      g.drawLine(x2,y2,result1_x,result1_y);
      g.drawLine(result1_x,result1_y, result2_x,result2_y);
      g.drawLine(result2_x,result2_y, result3_x,result3_y);
      g.drawLine(x2,y2,result3_x,result3_y);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /** 
   * Draw a hollow arrowhead (as lines) at the second end of the given line 
   * (this does not draw the line). This version provides good defaults for
   * the arrow size and shape.<p>
   *
   * @param int x1 first x coordinate of the line 
   * @param int y1 first y coordinate of the line 
   * @param int x2 second x coordinate of the line (arrow head point goes here)
   * @param int y2 second y coordinate of the line (arrow head point goes here)
   */
  public static void draw_arrowhead(Graphics2D g, int x1, int y1, int x2, int y2)
    {
      draw_arrowhead(g,x1,y1,x2,y2, 20, 15, 0.60);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Find the min and max points bounding the arrowhead that would be drawn
   * at the second point on the given line. <p> 
   * 
   * @param int x1 first x coordinate of the line 
   * @param int y1 first y coordinate of the line 
   * @param int x2 second x coordinate of the line (arrow head point goes here)
   * @param int y2 second y coordinate of the line (arrow head point goes here)
   * @param int arrow_head_len length of the sides of the arrow head
   * @param int arrow_head_angle angle between each side and the line in degrees
   * @param double arrow_inset percent inset for base of arrow [0..1]
   * @param Point min_pt returns holding the min x,y of the bound
   * @param Point max_pt returns holding the max x,y of the bound
   */
  public static void arrowhead_bound( int x1, int y1, int x2, int y2, 
    int arrow_head_len, int arrow_head_angle, double arrow_inset,
    Point min_pt, Point max_pt)
    {
      double dx, dy, len, sin_theta, cos_theta;
      double pt1_x, pt1_y, pt2_x, pt2_y, pt3_x, pt3_y;
      double ah_len = (double)arrow_head_len;
      double ah_angle = Math.PI*((double)arrow_head_angle)/180.0;
      int result1_x, result1_y, result2_x, result2_y, result3_x, result3_y;
      int max_x, max_y, min_x, min_y;

      /* figure out the length of the line */
      dx = (double)(x2-x1);
      dy = (double)(y2-y1);
      len = Math.sqrt(dx*dx+dy*dy);

      /* bail out now if its zero length */
      if (len == 0)
	{
	  min_pt.x = max_pt.x = x1;
	  min_pt.y = max_pt.y = y1;
	  return;
	}

      /* compute arrow head points if were were on line on x axis */
      pt1_x = len - ah_len*Math.cos(ah_angle);
      pt1_y = ah_len*Math.sin(ah_angle);
      pt2_x = len - (len-pt1_x)*arrow_inset;
      pt2_y = 0;
      pt3_x = pt1_x;
      pt3_y = -pt1_y;

      /* compute sin and cos of rotation to get line from x axis current pos */
      sin_theta = dy / len;
      cos_theta = dx / len;

      /* rotate and translate to get our final points */
      result1_x = (int)(pt1_x*cos_theta - pt1_y*sin_theta + 0.5) + x1;
      result1_y = (int)(pt1_x*sin_theta + pt1_y*cos_theta + 0.5) + y1;
      result2_x = (int)(pt2_x*cos_theta - pt2_y*sin_theta + 0.5) + x1;
      result2_y = (int)(pt2_x*sin_theta + pt2_y*cos_theta + 0.5) + y1;
      result3_x = (int)(pt3_x*cos_theta - pt3_y*sin_theta + 0.5) + x1;
      result3_y = (int)(pt3_x*sin_theta + pt3_y*cos_theta + 0.5) + y1;

      /* compute min and max */
      min_x = x2;
      if (min_x > result1_x) min_x = result1_x;
      if (min_x > result2_x) min_x = result2_x;
      if (min_x > result3_x) min_x = result3_x;
      min_y = y2;
      if (min_y > result1_y) min_y = result1_y;
      if (min_y > result2_y) min_y = result2_y;
      if (min_y > result3_y) min_y = result3_y;
      max_x = x2;
      if (max_x < result1_x) max_x = result1_x;
      if (max_x < result2_x) max_x = result2_x;
      if (max_x < result3_x) max_x = result3_x;
      max_y = y2;
      if (max_y < result1_y) max_y = result1_y;
      if (max_y < result2_y) max_y = result2_y;
      if (max_y < result3_y) max_y = result3_y;

      /* write back results */
      min_pt.x = min_x; min_pt.y = min_y;
      max_pt.x = max_x; max_pt.y = max_y;
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

  /**
   * Find the min and max points bounding the arrowhead that would be drawn
   * at the second point on the given line. <p> 
   * 
   * @param int x1 first x coordinate of the line 
   * @param int y1 first y coordinate of the line 
   * @param int x2 second x coordinate of the line (arrow head point goes here)
   * @param int y2 second y coordinate of the line (arrow head point goes here)
   * @param Point min_pt returns holding the min x,y of the bound
   * @param Point max_pt returns holding the max x,y of the bound
   */
  public static void arrowhead_bound( int x1, int y1, int x2, int y2, 
    Point min_pt, Point max_pt)
    {
      arrowhead_bound(x1,y1,x2,y2,20,15,0.60,min_pt,max_pt);
    }

  /* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

}
