
package org.remote.roadhog;

import java.awt.Point;
import java.awt.geom.Point2D;
import java.io.PrintStream;
import java.text.*;
import java.util.StringTokenizer;
import java.util.Date;
import org.w3c.dom.*;

/**
 * Represents a GPS waypoint.
 */
public class Waypoint extends XMLObject
{
	public Point2D.Double coord;
	public int x;
	public int y;
	public long time;
	public int speed;
	public int heading;
	public volatile Point mapPixel;
	public Point hack;

	private static SimpleDateFormat garminDF = new SimpleDateFormat("yyyy/MM/dd-HH:mm:ss");
	private static StringBuffer formatHelper = new StringBuffer ();

	/**	
	 * Creates an empty waypoint.
	 */
	public Waypoint ()
	{
		/* default */
	}
	
	/**	
	 * Creates a new waypoint from a line of text, as generated by the 
	 * "download track" option in Randolph Bentson's "gd2" utility; example:
	 * <pre>
	 * TRK N57 48.098831 W5 40.293789 2001/03/24-16:42:17 1
	 * TRK N57 48.097544 W5 40.284777 2001/03/24-16:42:18 1
	 * TRK N57 48.097544 W5 40.282202 2001/03/24-16:42:19 1
	 * TRK N57 48.080807 W5 40.212679 2001/03/24-16:42:35 1
	 * </pre>
	 */
	public Waypoint (String line)
	throws ParseException
	{
		StringTokenizer tok = new StringTokenizer (line);

		if (!("TRK".equals (tok.nextToken ()))) 
			throw new ParseException ("Line must begin with TRK");

		String latDeg = tok.nextToken ();
		String latMin = tok.nextToken ();
		String longDeg = tok.nextToken ();
		String longMin = tok.nextToken ();

		coord = new Point2D.Double ();

		try 
		{
			coord.y = Integer.parseInt (latDeg.substring (1));
			coord.y += Double.parseDouble (latMin) / 60.0;
			if (latDeg.startsWith("S")) coord.y *= -1;
		}
		catch (NumberFormatException nfe)
		{
			throw new ParseException ("Bad number format in latitude");
		}

		try 
		{
			coord.x = Integer.parseInt (longDeg.substring (1));
			coord.x += Double.parseDouble (longMin) / 60.0;
			if (longDeg.startsWith("W")) coord.x *= -1;
		}
		catch (NumberFormatException nfe)
		{
			throw new ParseException ("Bad number format in longitude");
		}

		time = garminDF.parse (tok.nextToken (), new ParsePosition (0)).getTime ();
	}

	/**	
	 * Creates a new waypoint from an XML node.
	 */
	public Waypoint (Node node)
	throws ParseException
	{
		NodeList sublist;
		Element waypoint = (Element) node;
		coord = new Point2D.Double ();

		try
		{
			sublist = waypoint.getElementsByTagName ("latitude");
			coord.y = Double.parseDouble (getTextChild (sublist.item(0)));

			sublist = waypoint.getElementsByTagName ("longitude");
			coord.x = Double.parseDouble (getTextChild (sublist.item(0)));

			sublist = waypoint.getElementsByTagName ("time");
			time = Long.parseLong (getTextChild (sublist.item(0)));

			sublist = waypoint.getElementsByTagName ("speed");
			if (sublist.getLength () > 0) speed = Integer.parseInt (getTextChild (sublist.item(0)));

			sublist = waypoint.getElementsByTagName ("heading");
			if (sublist.getLength () > 0) heading = Integer.parseInt (getTextChild (sublist.item(0)));
			sublist = waypoint.getElementsByTagName ("hack");
			if (sublist.getLength () > 0) hack = parsePoint ((Element) sublist.item (0));
		}
		catch (NumberFormatException nfe)
		{
			throw new ParseException ("Number format error");
		}
	}
	
	public String prettyLatitude ()
	{
		formatHelper.setLength (0);
		formatHelper.append ((coord.y < 0) ? 'S' : 'N');
		double absval = Math.abs (coord.y);
		return (prettyNumber (absval));
	}
	public String prettyLongitude ()
	{
		formatHelper.setLength (0);
		formatHelper.append ((coord.x < 0) ? 'W' : 'E');
		double absval = Math.abs (coord.x);
		if (absval < 100) formatHelper.append ('0');
		return (prettyNumber (absval));
	}

	public String prettyNumber (double absval)
	{
		int deg = (int) absval;
		int min = (int) ((absval - deg) * 6000.0);
		int frac = min % 100;
		min /= 100;	
		if (deg<10) formatHelper.append ('0');
		formatHelper.append (deg);
		formatHelper.append ('');
		if (min < 10) formatHelper.append ('0');
		formatHelper.append(min);
		formatHelper.append('.');
		if (frac < 10) formatHelper.append ('0');
		formatHelper.append(frac);
		formatHelper.append('\'');
		return (formatHelper.toString ());
	}

	public void toXML (PrintStream o, String indent)
	{
		o.println (indent + "<waypoint>");
		o.println (indent + "   <latitude>" + coord.y + "</latitude>");
		o.println (indent + "   <longitude>" + coord.x + "</longitude>");
		o.println (indent + "   <time>" + time + "</time>");
		o.println (indent + "   <speed>" + speed + "</speed>");
		o.println (indent + "   <heading>" + heading + "</heading>");
		if (hack != null)
		{
			o.println (indent + "   <hack><x>" + hack.x + "</x><y>" + hack.y + "</y></hack>");
		}
		o.println (indent + "</waypoint>");
	}

	public void determineMapPixel (Map aMap)
	{
		if (mapPixel == null) mapPixel = new Point ();
		aMap.mapGeoToPixel (coord, mapPixel);
		aMap.applyCorrection (mapPixel);
		if (hack != null) mapPixel.translate (hack.x, hack.y);
	}
}
